#include <struct.h>

//------------------------------------------------------------------
// Cylinder Tool goes here.

DWORD CALLBACK CylinderPreMode(tProcParams* params)
{
  CurrentState.SetState(
    Z3D_STATE_WAITFOR_FIRSTCLICK);
  long nSides = 16;
  params->sysp(Z3D_REQUEST_SETNUMERICPARAMETER, NULL, " Sides", &nSides, NULL);

  long nSegments = 1;
  long nBar = 2;
  params->sysp(Z3D_REQUEST_SETNUMERICPARAMETER, NULL, " Segments", &nSegments, &nBar);
  return 0;
}


DWORD CALLBACK CylinderOnRButtonDown(tProcParams* params)
{
  CurrentState.SetState(Z3D_STATE_WAITFOR_FIRSTCLICK);
  return 0;
}


DWORD CALLBACK CylinderOnLButtonDown(tProcParams* params)
{
  // the tool doesn't support Mapper view.
  if (params->ViewDesc->Kind == Z3D_VIEWMODE_MAPPER)
    return 0;

  // if we are still expecting first click,
  // then we've got it. Now wait for second one.
  if (!CurrentState.IsState(
    Z3D_STATE_WAITFOR_SECONDCLICK))
  {
    CurrentState.SetState(
      Z3D_STATE_WAITFOR_SECONDCLICK);
    return 0;
  }

  //Ok, create the sphere.
  long  nSegments  = 0;
  long  nSides    = 16;
  long  Vertices  = 0;
  long  Faces    = 0;

  long nBar = 1;
  nSides = params->sysp(Z3D_REQUEST_GETNUMERICPARAMETER, NULL, NULL, NULL, NULL);
  nSegments = params->sysp(Z3D_REQUEST_GETNUMERICPARAMETER, NULL, NULL, &nBar, NULL);

  char strName[MAX_OBJECT_NAMELENGTH];
  wsprintf(strName, "Cylinder");
  if (IDOK != params->sysp(Z3D_REQUEST_ENTERNAME,
    params->Objects,
    (char*)&strName[0],
    &nSides,
    &nSegments))
  {
    CurrentState.SetState(
      Z3D_STATE_WAITFOR_FIRSTCLICK);
    return Z3D_PLUGRESULT_REDRAWTHIS;
  }

  // check parameters:
  if (nSides < 3)
    nSides = 3;
  if (nSegments < 0)
    nSegments = 0;

  tPOINT  Center = (*params->LastPressedAt);
  float  fRadius = Dist3D(
    params->ProcessedLocation->x,
    params->ProcessedLocation->y,
    params->ProcessedLocation->z,
    params->LastPressedAt->x,
    params->LastPressedAt->y,
    params->LastPressedAt->z);
  if (fRadius < MINFLOAT)
  {
    AfxMessageBox("Too small Cylinder!");
    return 0;
  }

  tObject* pObj = new tObject((char*)&strName[0],
    (nSegments+2)*nSides+2,
    (nSegments+2)*nSides*2);
  pObj->mTransform._41 = Center.x;
  pObj->mTransform._42 = Center.y;
  pObj->mTransform._43 = Center.z;
  
  // lowest single vertex.
  pObj->VertTable->Table[Vertices].X = Center.x;
  pObj->VertTable->Table[Vertices].Y = Center.y - fRadius;
  pObj->VertTable->Table[Vertices].Z = Center.z;
  pObj->VertTable->Table[Vertices].NormalX = 0.0f;
  pObj->VertTable->Table[Vertices].NormalY = -1.0f;
  pObj->VertTable->Table[Vertices].NormalZ = 0.0f;
  Vertices++;
  // upper single vertex.
  pObj->VertTable->Table[Vertices].X = Center.x;
  pObj->VertTable->Table[Vertices].Y = Center.y + fRadius;
  pObj->VertTable->Table[Vertices].Z = Center.z;
  pObj->VertTable->Table[Vertices].NormalX = 0.0f;
  pObj->VertTable->Table[Vertices].NormalY = 1.0f;
  pObj->VertTable->Table[Vertices].NormalZ = 0.0f;
  Vertices++;

  for (long i = 0; i < 2+nSegments; i++)
    for (long j = 0; j < nSides; j++)
    {//create for
      pObj->VertTable->Table[Vertices].X = Center.x +
        (float)(cos(fdivide(j*M_PI, nSides*0.5f)))*fRadius;
      pObj->VertTable->Table[Vertices].Y = Center.y - fRadius +
        fdivide(2.0f*fRadius*(i), 1+nSegments);
      pObj->VertTable->Table[Vertices].Z = Center.z +
        (float)(sin(fdivide(j*M_PI, nSides*0.5f)))*fRadius;
      pObj->VertTable->Table[Vertices].NormalX =
        (pObj->VertTable->Table[Vertices].X-Center.x)/fRadius;
      pObj->VertTable->Table[Vertices].NormalY = 0.0f;
      pObj->VertTable->Table[Vertices].NormalZ =
        (pObj->VertTable->Table[Vertices].Z-Center.z)/fRadius;
      Vertices++;
    }//create for

  for (i = 0; i < 1+nSegments; i++)
    for (long j = 0; j < nSides; j++)
    {
      pObj->FaceTable->Table[Faces].I1 = 2 + nSides*i+j;
      pObj->FaceTable->Table[Faces].I2 = 2 + nSides*i+((j+1)%nSides);
      pObj->FaceTable->Table[Faces].I3 = 2 + nSides*(i+1)+j;
      Faces++;
      pObj->FaceTable->Table[Faces].I1 = 2 + nSides*(i+1)+((j+1)%nSides);
      pObj->FaceTable->Table[Faces].I2 = 2 + nSides*(i+1)+j;
      pObj->FaceTable->Table[Faces].I3 = 2 + nSides*i+((j+1)%nSides);
      Faces++;
    }
  //top and bottom:
  for (i = 0; i < nSides; i++)
  {
    pObj->FaceTable->Table[Faces].I1 = 0;
    pObj->FaceTable->Table[Faces].I2 = 2 + (i+1)%nSides;
    pObj->FaceTable->Table[Faces].I3 = 2 + i;
    Faces++;
    pObj->FaceTable->Table[Faces].I1 = 1;
    pObj->FaceTable->Table[Faces].I2 = 2 + nSides*(nSegments+1) + i;
    pObj->FaceTable->Table[Faces].I3 = 2 + nSides*(nSegments+1) + (i+1)%nSides;
    Faces++;
  }

  Axis = Center;
  float angle = M_PI/2.0f;

  z3dLevels level = CurrentState.GetLevel();
  CurrentState.SetLevel(Z3D_LEVEL_OBJECTS);
  pObj->VertTable->MarkAll();
  if (IsSide(params->ViewDesc->Kind))
  {
    tPOINT axis = tPOINT(0.0f, 0.0f, 1.0f);
    pObj->Rotate(
      params->ViewDesc,
      angle,
      TRUE,
      FALSE,
      axis, Center);
  }
  if (IsFrontal(params->ViewDesc->Kind))
  {
    tPOINT axis = tPOINT(1.0f, 0.0f, 0.0f);
    pObj->Rotate(
      params->ViewDesc,
      angle,
      TRUE,
      FALSE,
      axis, Center);
  }
  pObj->VertTable->UnMarkAll();
  CurrentState.SetLevel(level);
  pObj->nRequire = Z3D_PLUGRESULT_REDRAW;

  Backup(Z3D_BACKUP_FORADDITION, "Create Cylinder", FALSE, params);

  params->Objects->AddObject(pObj);
  delete pObj;

  CurrentState.SetState(Z3D_STATE_WAITFOR_FIRSTCLICK);
  return Z3D_PLUGRESULT_VIEWSREDRAW;
}

DWORD CALLBACK CylinderOnMouseMove(tProcParams* params)
{
  // if we are still expecting first click,
  // then do nothing - user simply moves the
  // mouse currently.
  if (!CurrentState.IsState(
    Z3D_STATE_WAITFOR_SECONDCLICK))
    return 0;

  // the tool doesn's support UVMapper view
  if (params->ViewDesc->Kind == Z3D_VIEWMODE_MAPPER)
    return 0;

  // if we got here, we are expecting for the second
  // click. So, draw the circular shape:

  float  fRadius;
  int    iRadius;
  fRadius = Dist3D(
    params->ProcessedLocation->x,
    params->ProcessedLocation->y,
    params->ProcessedLocation->z,
    params->LastPressedAt->x,
    params->LastPressedAt->y,
    params->LastPressedAt->z);
  if (params->ViewDesc->Kind == Z3D_VIEWMODE_USER)
    iRadius = (int)(fRadius*(
      (params->ViewDesc->ClientRect.bottom - params->ViewDesc->ClientRect.top)/
      (2.0f*tan(params->ViewDesc->CamFOV/360.0f*M_PI)*params->ViewDesc->Zoom)));

  else
    iRadius = (int)(fRadius*params->ViewDesc->Zoom);
  POINT center = ConvertToScreen(
    params->LastPressedAt->x,
    params->LastPressedAt->y,
    params->LastPressedAt->z,
    params->ViewDesc);
  params->pdc->SelectStockObject(NULL_BRUSH);
  params->pdc->SelectObject((CPen*)Global.PenThinBlueCreate);

  params->pdc->Ellipse(
    center.x-iRadius,
    center.y-iRadius,
    center.x+iRadius,
    center.y+iRadius);
  params->pdc->SelectObject((CPen*)Global.PenThinBlack);
  return  Z3D_PLUGRESULT_MANUALDRAW |
      Z3D_PLUGRESULT_NOCOVER;
}
